@@ -1422,21 +1422,25 @@ _gnutls_recv_int(gnutls_session_t session, content_type_t type,
  * difference is that it accepts a GnuTLS session, and uses different
  * error codes.
  * Note that if the send buffer is full, send() will block this
- * function.  See the send() documentation for full information.  You
- * can replace the default push function by using
- * gnutls_transport_set_ptr2() with a call to send() with a
- * MSG_DONTWAIT flag if blocking is a problem.
- * If the EINTR is returned by the internal push function (the
- * default is send()) then %GNUTLS_E_INTERRUPTED will be returned. If
+ * function.  See the send() documentation for more information.  
+ *
+ * You can replace the default push function which is send(), by using
+ * gnutls_transport_set_push_function().
+ *
+ * If the EINTR is returned by the internal push function 
+ * then %GNUTLS_E_INTERRUPTED will be returned. If
  * %GNUTLS_E_INTERRUPTED or %GNUTLS_E_AGAIN is returned, you must
- * call this function again, with the same parameters; alternatively
+ * call this function again, with the exact same parameters; alternatively
  * you could provide a %NULL pointer for data, and 0 for
  * size. cf. gnutls_record_get_direction(). 
  *
  * Note that in DTLS this function will return the %GNUTLS_E_LARGE_PACKET
  * error code if the send data exceed the data MTU value - as returned
  * by gnutls_dtls_get_data_mtu(). The errno value EMSGSIZE
- * also maps to %GNUTLS_E_LARGE_PACKET.
+ * also maps to %GNUTLS_E_LARGE_PACKET. 
+ * Note that since 3.2.13 this function can be called under cork in DTLS
+ * mode, and will refuse to send data over the MTU size by returning
+ * %GNUTLS_E_LARGE_PACKET.
  *
  * Returns: The number of bytes sent, or a negative error code.  The
  *   number of bytes sent might be less than @data_size.  The maximum
@@ -1455,6 +1459,13 @@ gnutls_record_send(gnutls_session_t session, const void *data,
 
 		int ret;
 
+		if (IS_DTLS(session)) {
+			if (data_size + session->internals.record_presend_buffer.length >
+				gnutls_dtls_get_data_mtu(session)) {
+				return gnutls_assert_val(GNUTLS_E_LARGE_PACKET);
+			}
+		}
+
 		ret =
 		    _gnutls_buffer_append_data(&session->internals.
 					       record_presend_buffer, data,
@@ -1474,6 +1485,8 @@ gnutls_record_send(gnutls_session_t session, const void *data,
  * All queued records will be sent when gnutls_uncork() is called, or
  * when the maximum record size is reached.
  *
+ * This function is safe to use with DTLS after GnuTLS 3.2.13.
+ *
  * Since: 3.1.9
  **/
 void gnutls_record_cork(gnutls_session_t session)
@@ -1524,7 +1537,7 @@ int gnutls_record_uncork(gnutls_session_t session, unsigned int flags)
 						       record_presend_buffer.
 						       length);
 			}
-			while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
+			while (ret < 0 && (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED));
 		} else {
 			ret =
 			    gnutls_record_send(session,
